Package: RNAmodR
Type: Package
Title: Detection of post-transcriptional modifications in high
        throughput sequencing data
Version: 1.27.0
Date: 2025-09-23
Authors@R: c(person("Felix G.M.", 
                    "Ernst", 
                    email = "felix.gm.ernst@outlook.com", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0001-5064-0928")),
             person("Denis L.J.", 
                    "Lafontaine", 
                    email = "denis.lafontaine@ulb.ac.be", 
                    role = c("ctb", "fnd")))
Description: RNAmodR provides classes and workflows for loading/aggregation data
    from high througput sequencing aimed at detecting post-transcriptional 
    modifications through analysis of specific patterns. In addition, utilities
    are provided to validate and visualize the results. The RNAmodR package
    provides a core functionality from which specific analysis strategies can
    be easily implemented as a seperate package.
biocViews: Software, Infrastructure, WorkflowStep, Visualization,
        Sequencing
License: Artistic-2.0
Encoding: UTF-8
LazyData: false
Depends: R (>= 4.0), S4Vectors (>= 0.27.12), IRanges (>= 2.23.9),
        GenomicRanges, Modstrings
Imports: methods, stats, grDevices, matrixStats, BiocGenerics,
        Biostrings (>= 2.57.2), BiocParallel, txdbmaker,
        GenomicFeatures, GenomicAlignments, Seqinfo, rtracklayer,
        Rsamtools, BSgenome, RColorBrewer, colorRamps, ggplot2, Gviz
        (>= 1.31.0), reshape2, graphics, ROCR
Suggests: BiocStyle, knitr, rmarkdown, testthat, RNAmodR.Data
VignetteBuilder: knitr
Collate: 'RNAmodR.R' 'AllGenerics.R'
        'Gviz-ModifiedSequenceTrack-class.R' 'settings.R'
        'Modifier-utils.R' 'SequenceDataFrame-class.R'
        'normalization.R' 'SequenceData-class.R'
        'SequenceDataSet-class.R' 'SequenceDataList-class.R'
        'Modifier-class.R' 'ModifierSet-class.R'
        'Modifier-Inosine-class.R' 'Modifier-Inosine-viz.R'
        'Modifier-roc.R' 'Modifier-subset.R' 'Modifier-viz.R'
        'ModifierSet-comparison.R' 'ModifierSet-viz.R'
        'RNAmodR-external-functions.R' 'RNAmodR-summary.R'
        'SequenceData-coverage.R' 'SequenceData-end-pos.R'
        'SequenceData-normalized-end-pos.R' 'SequenceData-pileup.R'
        'SequenceData-viz.R' 'SequenceData-protected-end-pos.R'
        'SequenceData-stats.R' 'SequenceData-subset.R'
        'SequenceDataList-viz.R' 'SequenceDataSet-viz.R' 'utils-bam.R'
        'utils.R'
RoxygenNote: 7.3.3
BugReports: https://github.com/FelixErnst/RNAmodR/issues
URL: https://github.com/FelixErnst/RNAmodR
git_url: https://git.bioconductor.org/packages/RNAmodR
git_branch: devel
git_last_commit: bc88bd5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:45:36 UTC; biocbuild
Author: Felix G.M. Ernst [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5064-0928>),
  Denis L.J. Lafontaine [ctb, fnd]
Maintainer: Felix G.M. Ernst <felix.gm.ernst@outlook.com>
