# Generated by roxygen2: do not edit by hand

export("settings<-")
export(CoverageSequenceData)
export(CoverageSequenceDataFrame)
export(End3SequenceData)
export(End3SequenceDataFrame)
export(End5SequenceData)
export(End5SequenceDataFrame)
export(EndSequenceData)
export(EndSequenceDataFrame)
export(ModDNASequenceTrack)
export(ModInosine)
export(ModRNASequenceTrack)
export(ModSetInosine)
export(Modifier)
export(ModifierSet)
export(NormEnd3SequenceData)
export(NormEnd3SequenceDataFrame)
export(NormEnd5SequenceData)
export(NormEnd5SequenceDataFrame)
export(PileupSequenceData)
export(PileupSequenceDataFrame)
export(ProtectedEndSequenceData)
export(ProtectedEndSequenceDataFrame)
export(SequenceData)
export(SequenceDataList)
export(SequenceDataSet)
export(aggregate)
export(aggregateData)
export(bamfiles)
export(compare)
export(compareByCoord)
export(dataType)
export(findMod)
export(getAggregateData)
export(getData)
export(getDataTrack)
export(hasAggregateData)
export(labelByCoord)
export(mainScore)
export(modType)
export(modifications)
export(modifierType)
export(modify)
export(pileupToCoverage)
export(plotCompare)
export(plotCompareByCoord)
export(plotData)
export(plotDataByCoord)
export(plotROC)
export(replicates)
export(sequenceData)
export(sequences)
export(settings)
export(stats)
export(subsetByCoord)
export(validAggregate)
export(validModification)
exportClasses(CoverageSequenceData)
exportClasses(CoverageSequenceDataFrame)
exportClasses(DNAModifier)
exportClasses(End3SequenceData)
exportClasses(End3SequenceDataFrame)
exportClasses(End5SequenceData)
exportClasses(End5SequenceDataFrame)
exportClasses(EndSequenceData)
exportClasses(EndSequenceDataFrame)
exportClasses(ModInosine)
exportClasses(ModSetInosine)
exportClasses(Modifier)
exportClasses(ModifierSet)
exportClasses(NormEnd3SequenceData)
exportClasses(NormEnd3SequenceDataFrame)
exportClasses(NormEnd5SequenceData)
exportClasses(NormEnd5SequenceDataFrame)
exportClasses(PileupSequenceData)
exportClasses(PileupSequenceDataFrame)
exportClasses(ProtectedEndSequenceData)
exportClasses(ProtectedEndSequenceDataFrame)
exportClasses(RNAModifier)
exportClasses(SequenceDFrame)
exportClasses(SequenceData)
exportClasses(SequenceDataFrame)
exportClasses(SequenceDataList)
exportClasses(SequenceDataSet)
exportClasses(SequenceModDNAStringSetTrack)
exportClasses(SequenceModRNAStringSetTrack)
exportMethods("[")
exportMethods("seqtype<-")
exportMethods("settings<-")
exportMethods(Modifier)
exportMethods(ModifierSet)
exportMethods(SequenceData)
exportMethods(aggregate)
exportMethods(aggregateData)
exportMethods(bamfiles)
exportMethods(cbind)
exportMethods(compare)
exportMethods(compareByCoord)
exportMethods(conditions)
exportMethods(dataType)
exportMethods(findMod)
exportMethods(getAggregateData)
exportMethods(getData)
exportMethods(getDataTrack)
exportMethods(hasAggregateData)
exportMethods(labelByCoord)
exportMethods(mainScore)
exportMethods(modType)
exportMethods(modifications)
exportMethods(modifierType)
exportMethods(modify)
exportMethods(names)
exportMethods(pileupToCoverage)
exportMethods(plotCompare)
exportMethods(plotCompareByCoord)
exportMethods(plotData)
exportMethods(plotDataByCoord)
exportMethods(plotROC)
exportMethods(ranges)
exportMethods(rbind)
exportMethods(replicates)
exportMethods(seqinfo)
exportMethods(seqlevels)
exportMethods(seqnames)
exportMethods(seqtype)
exportMethods(sequenceData)
exportMethods(sequences)
exportMethods(settings)
exportMethods(stats)
exportMethods(subset)
exportMethods(subsetByCoord)
exportMethods(validAggregate)
exportMethods(validModification)
import(BiocGenerics)
import(GenomicRanges)
import(S4Vectors)
import(methods)
importClassesFrom(Gviz,DisplayPars)
importClassesFrom(Gviz,SequenceTrack)
importClassesFrom(IRanges,CharacterList)
importClassesFrom(IRanges,IRanges)
importClassesFrom(IRanges,IntegerList)
importClassesFrom(IRanges,LogicalList)
importClassesFrom(IRanges,PartitioningByEnd)
importClassesFrom(IRanges,PartitioningByWidth)
importClassesFrom(IRanges,SplitDataFrameList)
importClassesFrom(Rsamtools,BamFileList)
importClassesFrom(Rsamtools,PileupFiles)
importFrom(BSgenome,seqnames)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpisup)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,bpparam)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(BiocParallel,register)
importFrom(Biostrings,"seqtype<-")
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,seqtype)
importFrom(Biostrings,xscat)
importFrom(GenomicAlignments,coverage)
importFrom(GenomicAlignments,findOverlaps)
importFrom(GenomicAlignments,readGAlignments)
importFrom(GenomicFeatures,exonsBy)
importFrom(Gviz,AnnotationTrack)
importFrom(Gviz,DataTrack)
importFrom(Gviz,DisplayPars)
importFrom(Gviz,RNASequenceTrack)
importFrom(Gviz,SequenceTrack)
importFrom(Gviz,displayPars)
importFrom(Gviz,plotTracks)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IRanges)
importFrom(IRanges,IntegerList)
importFrom(IRanges,LogicalList)
importFrom(IRanges,PartitioningByEnd)
importFrom(IRanges,PartitioningByWidth)
importFrom(IRanges,SplitDataFrameList)
importFrom(Modstrings,ModDNAString)
importFrom(Modstrings,ModDNAStringSet)
importFrom(Modstrings,ModRNAString)
importFrom(Modstrings,ModRNAStringSet)
importFrom(Modstrings,combineIntoModstrings)
importFrom(Modstrings,fullName)
importFrom(Modstrings,shortName)
importFrom(RColorBrewer,brewer.pal)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(Rsamtools,BamFileList)
importFrom(Rsamtools,FaFile)
importFrom(Rsamtools,PileupParam)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,idxstatsBam)
importFrom(Rsamtools,pileup)
importFrom(Rsamtools,scanBamHeader)
importFrom(Rsamtools,scanFa)
importFrom(S4Vectors,splitAsList)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqlevelsInUse)
importFrom(Seqinfo,seqnames)
importFrom(colorRamps,matlab.like)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,title)
importFrom(matrixStats,rowSds)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(rtracklayer,GFF3File)
importFrom(stats,setNames)
importFrom(txdbmaker,makeTxDbFromGFF)
