context("show")
test_that("show:",{
  data(msi,package = "RNAmodR")
  data(psd,package = "RNAmodR")
  data(e5sd,package = "RNAmodR")
  expect_output(show(msi))
  expect_output(show(psd))
  expect_output(show(psd[[1]]))
  expect_output(show(SequenceDataSet(e5sd,psd)))
  expect_output(show(SequenceDataList(SequenceDataSet(e5sd,psd),e5sd,psd)))
})
